#include "MacroUtils.hpp"
#include "StringView.hpp"

#define ENUM_VALUE(x, value) x = value,
#
#define ENUM_CASE(x, name)                                                                                             \
    case name::x:                                                                                                      \
        return ArbUt::StringViewLiteral(#x);
#
#define ENUM_PARSE_CASE(x, name)                                                                                       \
    case ConstHash(#x):                                                                                                \
        return name::x;
#
#define ENUM_TRY_PARSE_CASE(x, name)                                                                                   \
    case ConstHash(#x):                                                                                                \
        out = name::x;                                                                                                 \
        return true;
#
#define ENUM_PARSE_CASE_INSENSITIVE(x, name)                                                                           \
    case ConstHashCI(#x):                                                                                              \
        return name::x;
#
#define ENUM_TRY_PARSE_CASE_INSENSITIVE(x, name)                                                                       \
    case ConstHashCI(#x):                                                                                              \
        out = name::x;                                                                                                 \
        return true;
#
#define ENUM_GET_HIGHEST(x, name)                                                                                      \
    if ((int64_t)name::x > highest) {                                                                                  \
        highest = (int64_t)name::x;                                                                                    \
    }
#
#define ENUM_GET_LOWEST(x, name)                                                                                       \
    if ((int64_t)name::x < lowest) {                                                                                   \
        lowest = (int64_t)name::x;                                                                                     \
    }
#
#define ARRAY_NAME(x, name) name::x,

/*!
  \def ENUM(name, type, values...)
  Generates an enum class inheriting from specified type with specified name. Also generates a useful helper class with
  static methods for use with the enum.
*/

#define ENUM_WITH_START_VALUE(name, type, startValue, values...)                                                       \
    enum class name : type {                                                                                           \
        MACRO_UTILS_FOR_EACH_WITH_VALUE(ENUM_VALUE, startValue + ___MACRO_UTILS_NARGS(values) - 1, values)             \
    };                                                                                                                 \
    class name##Helper {                                                                                               \
        inline static uint32_t constexpr ConstHash(char const* input) noexcept {                                       \
            return *input ? static_cast<uint32_t>(*input) + 33 * ConstHash(input + 1) : 5381;                          \
        }                                                                                                              \
        inline static constexpr char charToLower(const char c) noexcept {                                              \
            return (c >= 'A' && c <= 'Z') ? c + ('a' - 'A') : c;                                                       \
        }                                                                                                              \
        inline static uint32_t constexpr ConstHashCI(char const* input) noexcept {                                     \
            return charToLower(*input) ? static_cast<uint32_t>(charToLower(*input)) + 33 * ConstHashCI(input + 1)      \
                                       : 5381;                                                                         \
        }                                                                                                              \
                                                                                                                       \
    public:                                                                                                            \
        constexpr static ArbUt::StringViewLiteral ToString(name value) noexcept {                                      \
            switch (value) { MACRO_UTILS_FOR_EACH(ENUM_CASE, name, values) }                                           \
            return "out of bounds"_cnc;                                                                                \
        }                                                                                                              \
        constexpr static name Parse(const char* input, bool caseInsensitive = false) {                                 \
            if (caseInsensitive)                                                                                       \
                return ParseCaseInsensitive(input);                                                                    \
            switch (ConstHash(input)) { MACRO_UTILS_FOR_EACH(ENUM_PARSE_CASE, name, values) }                          \
            throw std::runtime_error("Invalid " #name " string.");                                                     \
        }                                                                                                              \
        constexpr static bool TryParse(const char* input, name& out, bool caseInsensitive = false) noexcept {          \
            if (caseInsensitive)                                                                                       \
                return TryParseCaseInsensitive(input, out);                                                            \
            switch (ConstHash(input)) { MACRO_UTILS_FOR_EACH(ENUM_TRY_PARSE_CASE, name, values) }                      \
            return false;                                                                                              \
        }                                                                                                              \
        static std::vector<name> GetValues() noexcept { return {MACRO_UTILS_FOR_EACH(ARRAY_NAME, name, values)}; }     \
        constexpr static name Last() noexcept { return name::MACRO_UTILS_GET_LAST(values); }                           \
        constexpr static name First() noexcept { return name::MACRO_UTILS_GET_FIRST(values); }                         \
        constexpr static name Highest() noexcept {                                                                     \
            int64_t highest = -9223372036854775807;                                                                    \
            MACRO_UTILS_FOR_EACH(ENUM_GET_HIGHEST, name, values)                                                       \
            return (name)highest;                                                                                      \
        }                                                                                                              \
        constexpr static name Lowest() noexcept {                                                                      \
            int64_t lowest = 9223372036854775807;                                                                      \
            MACRO_UTILS_FOR_EACH(ENUM_GET_LOWEST, name, values)                                                        \
            return (name)lowest;                                                                                       \
        }                                                                                                              \
                                                                                                                       \
    private:                                                                                                           \
        constexpr static name ParseCaseInsensitive(const char* input) {                                                \
            switch (ConstHashCI(input)) { MACRO_UTILS_FOR_EACH(ENUM_PARSE_CASE_INSENSITIVE, name, values) }            \
            throw std::runtime_error("Invalid " #name " string.");                                                     \
        }                                                                                                              \
        constexpr static bool TryParseCaseInsensitive(const char* input, name& out) noexcept {                         \
            switch (ConstHashCI(input)) { MACRO_UTILS_FOR_EACH(ENUM_TRY_PARSE_CASE_INSENSITIVE, name, values) }        \
            return false;                                                                                              \
        }                                                                                                              \
    };

#define ENUM(name, type, values...) ENUM_WITH_START_VALUE(name, type, 0, values);
